/*
    --------------------------------------------------------------------
                             RMF RDS Widget
                Copyright (c) 2007-2012 Jacek Jędrzejewski
                    http://www.jacek.jedrzejewski.name/
                          All rights reserved.
    --------------------------------------------------------------------

    This file is part of RMF RDS Widget.
*/

/**
 * Object for interaction with Last.fm and audioscobbler.
 */
var AS = new Base;
AS.extend({
    constructor: null,
    
    // scrobbling is turned on?
    active: false,
    
    // details about the current song
    song: {
        set: false,
        title: '',
        artist: '',
        album: ''
    },
    
    // data about current scrobble
    scrobbling: {
        started_at: 0,
        updated_at: 0,
        scrobbled: false,
    },
    
    // for storing call state
    call_response: null,
    // flag indicating whether show or not show notify about invalid session key
    reauth_notification: false,
    // UI callback used after invalidation session key
    disable_callback: null,
    
    // api fields
    call_url:   'http://ws.audioscrobbler.com/2.0/',
    api_key:    '199d2bf39d67cd9f3ef1c2d3f4599772', // don't steal it, lol
    api_secret: '497b25a84daaee00df3992353d6dd803', // not very secret, lol
    sk:         '', // session key
    token:      '', // one-time auth key
    username:   '', // current username
    
    /**
     * First step of last.fm auth.
     */
    getToken: function(){
        var that = this;
        
        this.call_response = null;
        this.reauth_notification = false;
        
        // get the token
        this.apiCall(
            'auth.getToken', 
            {}, 
            function(xml, text){
                that.token = $('token', xml)[0].childNodes[0].nodeValue;
                
                // save the token in case of closing a widget in the middle of auth process
                widget.setPreferenceForKey(that.token, 'as_token');
                
                // open auth url where user can grant access to his/her account
                var authUrl = 'http://www.last.fm/api/auth/?api_key='+that.api_key+'&token='+that.token;
                widget.openURL(authUrl);
                
                // response was ok
                that.call_response = true;
            },
            function(code, text){
                // response failed
                that.call_response = false;
            }
        );
    },
    
    /**
     * Second step of last.fm auth.
     */
    getSessionKey: function(callback){
        var that = this;
        
        // shouldn't happen but possible
        if(this.token.length == 0)
        {
            opera.postError('0x91 AS - No token for session.');
            return false;
        }
        widget.setPreferenceForKey('', 'as_token');
        
        this.call_response = null;
        
        // get the session
        this.apiCall(
            'auth.getSession', 
            { 
                'token': this.token
            }, 
            function(xml, text){
                that.username = $('name', xml)[0].childNodes[0].nodeValue;
                that.sk = $('key', xml)[0].childNodes[0].nodeValue;
                
                widget.setPreferenceForKey(that.sk, 'as_sk');
                widget.setPreferenceForKey(that.username, 'as_username');
                
                // response was ok
                that.call_response = true;
                
                // execute callback
                if(typeof(callback) == typeof(Function))
                {
                    callback();
                }
            },
            function(code, text){
                // response failed
                that.call_response = false;
                
                // execute callback
                if(typeof(callback) == typeof(Function))
                {
                    callback();
                }
            }
        );
        
        this.token = '';
    },
    
    /**
     * Removes account connection and disables scrobbling.
     */
    disable: function(){
        // clear all preferences
        widget.setPreferenceForKey('', 'as_sk');
        widget.setPreferenceForKey('', 'as_username');
        widget.setPreferenceForKey('', 'as_token');
        
        // disable scrobbling
        AS.active = false;
    },
    
    // interval handler
    _nowPlayingInterval: false,
    
    /**
     * Sets a song for scrobbling. If previous song's scrobble was valid - sends it.
     */
    setSong: function(title, artist, album){
        // is there a previous song?
        if(this.song.set)
        {
            clearInterval(this._nowPlayingInterval);
            
            // isn't is scrobbled?
            if(!this.scrobbling.scrobbled)
            {
                // scrobble when the song was longer than 30 seconds
                if(this.scrobbling.updated_at - this.scrobbling.started_at > 30)
                {
                    this.scrobble(1);
                }
            }
        }
        
        // clear scrobble status
        this.scrobbling.scrobbled = false;
        
        //opera.postError('Set song: '+artist+' - '+title);
    
        this.song.title = title;
        this.song.artist = artist;
        this.song.album = album;
        this.song.set = true;
        
        var d = new Date();
        
        // set scrobble time
        this.scrobbling.started_at = parseInt(d.getTime()/1000);
        this.scrobbling.updated_at = parseInt(d.getTime()/1000);
        
        // update profile "now playing" song
        this.updateNowPlaying();
        
        // counter is used for "now playing" updater to update for each 4 seconds
        var counter = 0;
        
        var that = this;
        
        // interval is used to update the length of the playback
        this._nowPlayingInterval = setInterval(function(){
            var d = new Date();
            
            that.scrobbling.updated_at = parseInt(d.getTime()/1000);
            
            // update "now playing" for each 4 seconds
            if(counter >= 4)
            {
                that.updateNowPlaying();
                counter = 0;
            }
            
            // scrobble after 4 minutes of playback
            if(that.scrobbling.updated_at - that.scrobbling.started_at >= 4*60 && !that.scrobbling.scrobbled)
            {
                that.scrobble(2);
            }
            
            counter++;
        }, 1000);
    },
    
    /**
     * Clears current song.
     */
    clearSong: function(){
        this.song.title = '';
        this.song.artist = '';
        this.song.album = '';
        this.song.set = false;
        this.clearScrobbling();
        
        clearInterval(this._nowPlayingInterval);
    },
    
    /**
     * Clears current scrobble.
     */
    clearScrobbling: function(){
        this.scrobbling.started_at = 0;
        this.scrobbling.updated_at = 0;
        this.scrobbling.scrobbled = false;
    },
    
    /**
     * Updates profile's "now playing" song. It's not yet a scrobble.
     */
    updateNowPlaying: function(){
        // do not try to call api without scrobbling enabled
        if(!this.active)
        {
            return false;
        }
    
        this.apiCall(
            'track.updateNowPlaying',
            {
                'track': this.song.title,
                'artist': this.song.artist,
                'album': this.song.album,
                'sk': this.sk
            },
            function(xml, text){
            },
            function(code, text){
            }
        );
    },
    
    scrobble: function(type){
        // do not try to call api without scrobbling enabled
        if(!this.active)
        {
            return false;
        }
        
        this.apiCall(
            'track.scrobble',
            {
                'track[0]': this.song.title,
                'artist[0]': this.song.artist,
                'album[0]': this.song.album,
                'timestamp[0]': this.scrobbling.started_at,
                'sk': this.sk
            },
            function(xml, text){
                // debugging
                opera.postError('Scrobbled ('+type+')');
            }
        );
        
        // set a flag that current scrobble has been sent
        this.scrobbling.scrobbled = true;
    },
    
    /**
     * Makes last.fm api call.
     */
    apiCall: function(method, params, callback, callbackError){
        var that = this;
        
        try
        {
            var args = {};
            
            var $req = new XMLHttpRequest();
            
            var url = this.call_url;
            
            // arguments that are always required
            args['method'] = method;
            args['api_key'] = this.api_key;
            
            // add provided addidional call arguments
            if(typeof(params) == 'object' && Object.size(params) > 0)
            {
                for(var key in params)
                {
                    args[key] = params[key];
                }
            }
            
            // calculate signature of all arguments
            var sig = '';
            
            // sort lphabetically
            args = sort_object(args);
            
            for(var key in args)
            {
                sig += key;
                sig += args[key];
            }
            
            // signature is a md5 hash string
            var signature = hex_md5(sig+this.api_secret);
            
            // add signature argument
            args['api_sig'] = signature;
            
            // build query string made of arguments
            var argStr = build_query(args);
            
            $req.onreadystatechange = function(){
                if($req.readyState == 4)
                {
                    var xml = $req.responseXML;
                    
                    // check for proper api call
                    if(xml.documentElement.getAttribute('status') == "ok")
                    {
                        if(typeof(callback) == typeof(Function))
                        {
                            callback(xml, $req.responseText);
                        }
                    }
                    else
                    {
                        // status failed
                        
                        // log http status code
                        if($req.status != 200)
                        {
                            opera.postError('0x91 XHR AS API Call ('+method+'): '+$req.status+' '+$req.statusText);
                        }
                        
                        var code = parseInt($('error', xml)[0].getAttribute('code'));
                        var text = $('error', xml)[0].childNodes[0].nodeValue;
                        
                        opera.postError('0x91 XHR AS API Call ('+method+') failed ('+code+'): '+text);
                        
                        // invalid session key
                        if(code == 9)
                        {
                            // show notify
                            if(!that.reauth_notification)
                            {
                                widget.showNotification(I18N._('as_invalid_session'), function(){
                                    Rds.showSettings(true);
                                });
                                
                                that.reauth_notification = true;
                            }
                            
                            // disable scrobbling
                            that.disable();
                            
                            that.disable_callback();
                        }
                        
                        if(typeof(callback) == typeof(Function))
                        {
                            callbackError(code, text);
                        }
                    }
                }
            };
            
            $req.open('POST', this.call_url+'?'+argStr, true);
            $req.send(null);
        }
        catch(exception)
        {
            opera.postError('0x90 XHR AS API Call ('+method+') EXCEPTION: '+exception);
        }
    }
});