/*
    --------------------------------------------------------------------
                             RMF RDS Widget
                Copyright (c) 2007-2012 Jacek Jędrzejewski
                    http://www.jacek.jedrzejewski.name/
                          All rights reserved.
    --------------------------------------------------------------------

    This file is part of RMF RDS Widget.
*/

String.prototype.trim = function(){
    return this.replace(/^\s+|\s+$/g,"");
}
String.prototype.ltrim = function(){
    return this.replace(/^\s+/,"");
}
String.prototype.rtrim = function(){
    return this.replace(/\s+$/,"");
}
String.prototype.nl2br = function(){
    return this.replace(/([^>]?)\n/g, '$1<br />\n');
}

function html_entity_decode(str){
    var tarea = document.createElement('textarea');
    tarea.innerHTML = str; 
    var val = tarea.value;
    return val;
}

function html_entity_encode(str){
    str = str.replace(/"/g, '&quot;');
    str = str.replace(/&/g, '&amp;');
    str = str.replace(/>/g, '&gt;');
    str = str.replace(/</g, '&lt;');
    return str;
}

Element.prototype.hasClass = function(name){
    return this.className.match(new RegExp('(\\s|^)'+name+'(\\s|$)'));
}
Element.prototype.addClass = function(name){
    if(!this.hasClass(name))
    {
        this.className += " "+name;
    }
}
Element.prototype.removeClass = function(name){
    if (this.hasClass(name))
    {
        var reg = new RegExp('(\\s|^)'+name+'(\\s|$)');
        this.className = this.className.replace(reg, ' ');
    }
}

function in_array(val, arr){
    for(var i = 0, l = arr.length; i < l; i++)
    {
        if(arr[i] == val)
        {
            return true;
        }
    }
    return false;
}

function array_remove(arr, needle){
    for(i=0; i < arr.length; i++)
    {
        if(needle == arr[i])
        {
            arr.splice(i, 1);
        }
    }
}

function array_search(needle, haystack, arg_strict){
    var strict = !!arg_strict;
    var key = '';
 
    for(key in haystack)
    {
        if((strict && haystack[key] === needle) || (!strict && haystack[key] == needle))
        {
            return key;
        }
    }
 
    return false;
}

function findPosX(obj){
    var curleft = 0;
    
    if(obj.offsetParent)
    {
        while(1) 
        {
            curleft += obj.offsetLeft;
            if(!obj.offsetParent)
            {
                break;
            }
            obj = obj.offsetParent;
        }
    }
    else if(obj.x)
    {
        curleft += obj.x;
    }
    
    return curleft;
}

function versionCompare(first, second){
    var st = first.split('.');
    var nd = second.split('.');
    st[0] = parseInt(st[0]);
    st[1] = parseInt(st[1]);
    nd[0] = parseInt(nd[0]);
    nd[1] = parseInt(nd[1]);
    
    switch(true)
    {
        case st[0] > nd[0]: return 1;
        case st[0] < nd[0]: return -1;
        case st[0] == nd[0]: 
            switch(true)
            {
                case st[1] > nd[1]: return 1;
                case st[1] < nd[1]: return -1;
                case st[1] == nd[1]: return 0;
            }
    }
}

function extract_number(value){
    var n = parseInt(value);
    return n == null || isNaN(n) ? 0 : n;
} 

function build_query(args, start){
    start |= false;

    var q = '';
    
    for(key in args)
    {
        if(q.length)
        {
            q += "&";
        }
        else if(start)
        {
            q += '?';
        }
        
        q += key+"="+args[key];
    }
    
    return q;
}

function sort_object(arr){
    // Setup Arrays
    var sortedKeys = new Array();
    var sortedObj = {};

    // Separate keys and sort them
    for(var i in arr)
    {
        sortedKeys.push(i);
    }
    
    sortedKeys.sort();

    // Reconstruct sorted obj based on keys
    for(var i in sortedKeys)
    {
        sortedObj[sortedKeys[i]] = arr[sortedKeys[i]];
    }
    
    return sortedObj;
}

Object.size = function(obj){
    var size = 0, key;
    
    for(key in obj)
    {
        if(obj.hasOwnProperty(key))
        {
            size++;
        }
    }
    
    return size;
};
