/*
    --------------------------------------------------------------------
                             RMF RDS Widget
                Copyright (c) 2007-2012 Jacek Jędrzejewski
                    http://www.jacek.jedrzejewski.name/
                          All rights reserved.
    --------------------------------------------------------------------

    This file is part of RMF RDS Widget.
*/

var Scroller = Base.extend({
    areaHeight: 0,
    
    _barHeight: 0,
    _contentTop: 0,
    
    _buttonTimeout: false,
    
    movingBar: false,
    movingBarReleaseCallback: false,
    
    scrollingActive: false,
    scrollingActiveStopCallback: false,
    
    constructor: function($area, $scroll, $container, $content){
        this.$area = $area;
        this.$scroll = $scroll;
        this.$container = $container;
        this.$content = $content;
        this.$bar = $('span.bar', this.$scroll)[0];
        
        this.areaHeight = this.$container.offsetHeight;
        
        var callback = this;
        
        buttonUp = $('.up', this.$scroll)[0];
        buttonDown = $('.down', this.$scroll)[0];
        
        buttonUp.addEventListener('click', function(){return false;}, false);
        buttonUp.addEventListener('mousedown', function(e){
            if(e.button != 0) return false;
            callback.go('up'); 
            callback._buttonTimeout = setTimeout(function(){
                callback.go('up'); callback.clearScrollInterval('up');
            }, 500); 
            return false;
        }, false);
        buttonUp.addEventListener('mouseup', function(){ clearInterval(callback._buttonTimeout); }, false);
        buttonUp.addEventListener('mouseout', function(){ clearInterval(callback._buttonTimeout); }, false);
        
        buttonDown.addEventListener('click', function(){return false;}, false);
        buttonDown.addEventListener('mousedown', function(e){
            if(e.button != 0) return false;
            callback.go('down'); 
            callback._buttonTimeout = setTimeout(function(){
                callback.go('down'); callback.clearScrollInterval('down');
            }, 500); 
            return false;
        }, false);
        buttonDown.addEventListener('mouseup', function(){ clearInterval(callback._buttonTimeout); }, false);
        buttonDown.addEventListener('mouseout', function(){ clearInterval(callback._buttonTimeout); }, false);
        
        stopScrollingTimeout = false;
        var stopScrolling = function(){
            callback.scrollingActive = false;
            if(callback.scrollingActiveStopCallback !== false && typeof callback.scrollingActiveStopCallback == 'function')
            {
                callback.scrollingActiveStopCallback();
            }
        };
        
        this.$area.addEventListener('mousewheel', function(ev){
            ev.preventDefault();
            ev.stopPropagation();
            
            delta = ev.wheelDelta / 100;
            
            clearTimeout(stopScrollingTimeout);
            callback.scrollingActive = true;
            stopScrollingTimeout = setTimeout(stopScrolling, 50);
            
            if(delta)
            {
                factor = Math.floor(Math.abs(delta));
                
                if(delta < 0)
                {
                    callback.go('down', factor);
                }
                else if(delta > 0)
                {
                    callback.go('up', factor);
                }
            }
            
            return false;
        }, false);
        
        callback.$bar.style.top = '12px';
        
        var _startY = 0;
        var _offsetY = 0;
        var _dragElement;
        var _lastPos = null;
        
        this.$bar.addEventListener('mousedown', function(e){
            if(e.button != 0) return false;
            callback.movingBar = true;
            this.addClass('active');
            target = this;
            if(e.button == 0)
            {
                $('#lyrics')[0].onselectstart = function(){ return false; } 
                $('#lyrics')[0].onmouseup = function(){ return false; }
                $('#lyrics')[0].onmousedown = function(){ return false; }
    
                _startY = e.clientY;
                _lastPos = _startY;
                _offsetY = extract_number(target.style.top);
                _dragElement = target;
                
                document.onmousemove = function(e){
                    Rds.draggingActive = true;
                    
                    if(_dragElement == null)
                    {
                        return false;
                    }
                    
                    if(_lastPos == null)
                    {
                        _lastPos = parseInt(_startY);
                    }
                    
                    if(_lastPos != e.clientY)
                    {
                        var delta = _lastPos - e.clientY;
                        var pos = _offsetY+e.clientY-_startY;
                        
                        var top = parseInt(callback.$bar.style.top); 
                        var max = callback.areaHeight-callback._barHeight-12;
                        var stop = false;
                        
                        if((delta < 0 && pos >= top) || (delta > 0 && pos <= top))
                        {
                            top -= delta;
                            
                            if(top < 12)
                            {
                                top = 12;
                            }
                            
                            if(top > max)
                            {
                                top = max;
                            }
                            
                            callback.$bar.style.top = top+'px';
                            
                            top -= 12;
                            max -= 12;
                            
                            ratio = (top * callback.$content.offsetHeight) / (callback.areaHeight - 24);
                            
                            callback._contentTop = Math.floor((0-ratio) / 15) * 15;
                            
                            var max_height = 0 - callback.$content.offsetHeight + callback.areaHeight;
                            if(callback._contentTop < max_height)
                            {
                                callback._contentTop = max_height;
                            }
                            
                            callback.$content.style.top = callback._contentTop+'px';
                        }
                    }
                    _lastPos = parseInt(e.clientY);
                }
                
                document.body.focus();
                return false;
            }
        }, false);
        document.addEventListener('mouseup', function(e){
            if(_dragElement != null)
            {   
                callback.$bar.removeClass('active');
                callback.movingBar = false;
                
                if(callback.movingBarReleaseCallback !== false && typeof callback.movingBarReleaseCallback == 'function')
                {
                    callback.movingBarReleaseCallback();
                }
                
                _dragElement = null;
                _lastPos = null;
                document.onmousemove = null;
                
                callback.updatePosition(Math.abs(callback._contentTop));
                
                Rds.draggingActive = false;
                
                textSelecting('#lyrics');
            }
        }, false);
        
        this.reset();
    },
    
    clearScrollInterval: function(dir){
        var callback = this;
        
        clearTimeout(callback._buttonTimeout); 
        callback._buttonTimeout = setInterval(function(){ callback.go(dir) }, 100);
    },
    
    go: function(dir, factor){
        if(this._barHeight == 0) return false;
        
        factor |= 1;
        
        if(factor < 1)
        {
            factor = 1;
        }
        
        prevTop = this._contentTop;
        
        if(dir == 'down')
        {
            this._contentTop -= 15*factor;
            var max_height = 0 - this.$content.offsetHeight + this.areaHeight;
            if(this._contentTop < max_height)
            {
                this._contentTop = max_height;
            }
        }
        else if(dir == 'up')
        {
            this._contentTop += 15*factor;
            if(this._contentTop > 0)
            {
                this._contentTop = 0;
            }
        }
        else
        {
            this._contentTop = 0;
        }
        if(prevTop == this._contentTop)
        {
            return true;
        }
        
        this.$content.style.top = this._contentTop+'px';
        
        this.updatePosition(Math.abs(this._contentTop));
    },
    
    reset: function(){
        this.go('reset');
        if(this.$content.offsetHeight < this.areaHeight)
        {
            this._barHeight = 0;
            this.$bar.style.display = 'none';
            $('span.bar span.body', this.$scroll)[0].style.height = '0';
        }
        else
        {
            this._barHeight = Math.round((this.areaHeight * (this.areaHeight - 24)) / this.$content.offsetHeight);
            this.$bar.style.display = 'block';
            $('span.bar span.body', this.$scroll)[0].style.height = (this._barHeight-4)+'px';
        }
        
        this.$bar.style.height = this._barHeight+'px';
    },
    
    updatePosition: function(top){
        this.$bar.style.top = 12 + Math.round(top / this.$content.offsetHeight * (this.areaHeight - 24)) + 'px';
    },
    
    scrollTo: function(top){
        this._contentTop = top;
        if(this._contentTop > 0)
        {
            this._contentTop = 0;
        }
        
        var max_height = 0 - this.$content.offsetHeight + this.areaHeight;
        if(this._contentTop < max_height)
        {
            this._contentTop = max_height;
        }
        
        this.$content.style.top = this._contentTop+'px';
        
        this.updatePosition(Math.abs(this._contentTop));
    }
});